/**
 * @file Defines components that are used to interact with the transcription proxy servlet.
 * @author Carl Stephens
 * @module
 */

import { log } from "./Util"

export class MacronRestorationError extends Error {
    /**
     * Initialises a new instance of the {@link MacronRestorationError} object.
     *
     * @param {Number | undefined} statusCode The HTTP status code of the error.
     * @param {String | undefined} message The status message.
     * @param {String | null} fileName The file on which the transcription failed.
     */
    constructor(message = undefined, fileName = null, statusCode = -1) {
        super(message);

        /** @type {String | null} The name of the file that the transcription error occured on. */
        this.fileName = fileName;

        /** @type {Number | undefined} The status code returned by the API when the erro was generated. */
        this.statusCode = statusCode;
    }
}

/**
 * A service that performs requests to the macron restoration API.
 */
export default class MacronRestorationService {
    constructor() {
        /** @type {String} The URL to which query POST requests should be made. */
        if (process.env.NODE_ENV !== "production") {
            this.rootQueryUrl = "//localhost:8383/gs3-macroniser/";
        }
        else {
            this.rootQueryUrl = "/gs3-macroniser/";
        }
    }

    /**
     * Performs a request to the macron restoration API to restore the given input text.
     * @param {String} input The input text to macronise
     * @returns {Promise<{ w?: String, macronised?: Boolean, linebreaks?: Number }>} The JSON response from the macron restoration API.
     */
    async directMacronisation(input, preserveExistingMacrons) {
        const queryUrl = this.rootQueryUrl + "direct";

        const formData = new FormData();
        formData.append("fragment", input);
        formData.append("preserveExistingMacrons", preserveExistingMacrons);

        try {
            const response = await fetch(
                queryUrl,
                {
                    method: "POST",
                    body: formData
                }
            );

            if (!response.ok) {
                log(`Macron Restoration API failed with status ${response.status} and message ${response.statusText}`, "error")
                throw new MacronRestorationError(response.statusText, undefined, response.status);
            }

            return await response.json();
        }
        catch (e) {
            log(`Macron restoration failed with reason ${e}`, "error");
            throw new MacronRestorationError(undefined, "Unknown");
        }
    }

    /**
     * Performs a query to macronise the given file.
     *
     * @param {File} file The file to macronise.
     * @param {Boolean} preserveExistingMacrons Indicates if existing macrons on the file should be preserved.
     * @returns {Promise<{ fileName: String, filePath: String, fileType: String }>} Information about the macronised file.
     * @throws {MacronRestorationError} When the macronisation request fails to complete.
     */
    async fileMacronisation(file, preserveExistingMacrons) {
        const queryUrl = this.rootQueryUrl + "file";
        const formData = new FormData();

        formData.append("o", "json");
        formData.append("preserveExistingMacrons", preserveExistingMacrons);
        formData.append(file.name, file, file.name);

        try {
            const response = await fetch(
                queryUrl,
                {
                    method: "POST",
                    body: formData
                }
            );

            if (!response.ok) {
                log(`Macronisation API failed with status ${response.status} and message ${response.statusText}`, "error")
                throw new MacronRestorationError(response.statusText, file.Name, response.status);
            }

            return await response.json();
        }
        catch (e) {
            log(`Macronisation failed with reason ${e}`, "error");
            throw new MacronRestorationError("Unknown", file.Name, "Unknown");
        }
    }

    /**
     * Builds a URL used to retrieve a file.
     * @param {String} filePath The path of the file to download.
     * @param {String} fileName The name to display in the download response.
     */
    buildDownloadUrl(filePath, fileName) {
        return `${this.rootQueryUrl}download?filepath=${filePath}&filename=${fileName}`;
    }
}
